#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Tools>Custom Font Labels'
#
# The first option creates new letter faces from over-sized visible layer/s
# The second option clears the contentes of the old character layer before copying and pasting the new letter face
# The third and fourth options set the mode (legacy and new) to be used when copying and pasting to generate the new font face
# The fifth option removes non-font character layers from the image - apart from any layer name containing the word 'legend'
# The sixth option will re-position all layers to the origin of their parent image (Option 4 does this too)
# The seventh option will re-order the layers in the custom font file according to the following order
#		space.png (or  .ping)
#		Capital letters (A-Z)
#		Lower case letters (a-z)
#		Numbers (0-9)
#		Basic maths characters
# 		Punctuation characters
#		Other keyboard characters
#		Special characters (Pound Sterling sign, Copyright Sign, Registered Trademark sign)
# You can change this order by altering the appropriate lines (currently beginning at line no 130) of the plug-in
# The eighth option will display a message box listing all the characters (ie layers) defined in the custom font
# The ninth option defines the string to be used to append to single letter layer and/or path names
# The tenth option will add .png to all the layers with one character names ie  A to A.png
# The eleventh option will add .png to all the paths with one character names
# The twelfth option will combine the characters from the xcf custom font file selected, with the file selection tool below, with the current characters
# The thirteenth option is the xcf file selection tool

# Version 0.01 Fixed adding .png to single letter layers and paths with special characters ie unicode characters
# Version 1.02 Adds an option to create a new face for a custom font from additional/decorative visible layers
# Version 1.03 Re-position space layer, if one exists, along with other letter layers and adds the option to remove non-font-character layers from the image
# Version 1.04 Trim and feather pixels from letter faces to remove halo effect
# Version 1.05 Fixed bug where character layer is not at image origin (and so doesn't copy new face correctly): Adds option to use different modes when copying from visible layers to font face
# Version 1.06 Added option to re-position all layers to the origin of their parent image ie with x and y co-ordinates set to zero: Re-organized dialog and filter structure so that all options can be set without any issues
# Version 1.07 Added option to clear the existing letter face before copying: Re-ordered flow of filter (yet again); Added options to combine two custom font files
# Version 1.08 Fixed potential bug when layer is bigger than image
#              Changed filter location
# Version 1.09 Added mode choice when combining two custom fonts

from gimpfu import *
from array import array
from gtk import gdk
import math
import random

w = gdk.screen_width()
h = gdk.screen_height()

# Some global colours we may need to use

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	


def do_stuff(image, layer, do_new_letters, clear_contents, old_mode, new_mode, do_remove_non_letter_layers, do_place_at_origin, do_sort, do_layer_info, text_string, do_layer_names, do_path_names, do_merge_files, xcffilename):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
		

    if do_layer_names:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_list = image.layers
        loop = 0
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_list[loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1:
                layer_name += text_string
                pdb.gimp_item_set_name(layer_list[loop],layer_name)
            else:
                if layer_name == "space":
                    layer_name += text_string
                    pdb.gimp_item_set_name(layer_list[loop],layer_name)
            loop += 1
			
			
    if do_path_names:
        num_paths, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        loop = 0
        while loop < num_paths:
            path_name = pdb.gimp_item_get_name(vector_list[loop])
            path_name = path_name.decode('utf-8')
            if len(path_name) == 1:
                path_name += text_string
                pdb.gimp_item_set_name(vector_list[loop],path_name)
            else:
                if path_name == "space":
                    path_name += text_string
                    pdb.gimp_item_set_name(vector_list[loop],path_name)
            loop += 1				
			
			
    if do_sort:
        space_case = [' ','space']
        upper_case = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']
        lower_case = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        numerals = ['9','8','7','6','5','4','3','2','1','0']
        mathematical = ['-','+','=','/','*']
        punctuation = ['.',',',':',';','!','?','\'','\"']
        other_characters = ['$','%','^','&','_','{','}','@','#','~','|','\\','<','>','(',')','[',']','{','}']
        special_characters = unichr(163) + unichr(169) + unichr(174) # Pound Sterling sign, Copyright Sign, Registered Trademark sign
		
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_id_list = image.layers
        loop = 0
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_id_list[loop])
            if loop == 0:
                layer_name_list = [layer_name]
            else:
                layer_name_list.append(layer_name)
            loop += 1

        all_layers = layer_name_list
		
        for character in special_characters:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
								
        for character in other_characters:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
								
        for character in punctuation:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
						
        for character in mathematical:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in numerals:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in lower_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in upper_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
			
        for character in space_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
 
    if do_place_at_origin:
	
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
					
        loop = 0
        # Get the list of layer ids
        layer_names_list = image.layers
        
        while loop < num_layers:
            # make sure that existing layers are positioned at the origin of the image
            layer_names_list[loop].set_offsets(0, 0)
            loop += 1		

			
    if do_new_letters:
        mode = fn_get_mode(old_mode, new_mode)
        im_width = image.width
        im_height = image.height
    
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
		
		
        visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
        
        while visi_loop < num_layers:
            # check for layers to create new faces for
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": has_a_space = True
            if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                if first_letter_layer < 0: first_letter_layer = visi_loop
            else:
                pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                is_empty = pdb.gimp_selection_is_empty(image)
                if is_empty == False: other_layers += 1	
                    		
            visi_loop += 1
			
        visi_loop = 0
        
        while visi_loop < num_layers and other_layers > 0:
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": 
                has_a_space = True
                space_loop = visi_loop
            if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                # turn off visibility of letter layers
                pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)  
            visi_loop += 1
			
			
        if other_layers > 0:
	
            visi_loop = 0
            while visi_loop < num_layers:
                visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
                visi_name = visi_name.decode('utf-8')
                if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                    # make sure that existing layers are positioned at the origin of the image
                    layer_names_list[visi_loop].set_offsets(0, 0)
                    # prepare to move layer to random position within the confines of the image size
                    layer_width = layer_names_list[visi_loop].width
                    if layer_width > im_width: layer_width = im_width
                    layer_height = layer_names_list[visi_loop].height
                    if layer_height > im_height: layer_height = im_height
                    random.seed()
                    random_x = random.randrange(0, (im_width + 1) - layer_width)
                    random.seed()
                    random_y = random.randrange(0, (im_height + 1) - layer_height)
					
                    # make a duplicate of the current character layer and insert
                    layer_position = pdb.gimp_image_get_item_position(image, layer_names_list[visi_loop])
                    layer_copy = pdb.gimp_layer_copy(layer_names_list[visi_loop], True)
                    pdb.gimp_image_insert_layer(image, layer_copy, None, layer_position)
                    # set random position to copy visible layers 
                    layer_copy.set_offsets(random_x, random_y)
                    
				
                    # select the contents of the layer and clear contents
                    pdb.gimp_image_select_item(image, 2, layer_copy)
                    pdb.gimp_drawable_edit_clear(layer_copy)
					
					
                    # copy the contents of the visible layers into the selection
                    success = pdb.gimp_edit_copy_visible(image)
				
                    # turn on layer visibility
                    pdb.gimp_item_set_visible(layer_copy, True) 
					
                    # paste the selection into the floating layer
                    floating_sel = pdb.gimp_edit_paste(layer_copy,True)
                    # and anchor it
                    pdb.gimp_floating_sel_anchor(floating_sel)
                    # set the layer mode as per the dialog UI when the contents of the original character layer have not been cleared
                    if not clear_contents: pdb.gimp_layer_set_mode(layer_copy, mode)
                    # Re-position the copied layer
                    layer_copy.set_offsets(0, 0)
                    # Make the original character layer visible before we merge down
                    pdb.gimp_item_set_visible( layer_names_list[visi_loop], True)
                    # option to clear the original character
                    if clear_contents: 
                        pdb.gimp_selection_none(image)
                        pdb.gimp_drawable_edit_clear(layer_names_list[visi_loop])
                    # Merge down the copied layer to the character layer
                    pdb.gimp_image_merge_down(image, layer_copy, 0)
                    # Re-set the id of the layer
                    layer_names_list[visi_loop] = pdb.gimp_image_get_active_layer(image)
                    # Remove annoying halo
                    proc_trim_away(image, layer_names_list[visi_loop], 1)
                    # Turn off layer visibility
                    pdb.gimp_item_set_visible(layer_names_list[visi_loop], False) 
				
				
                
                visi_loop += 1
				
                if has_a_space:
                    # re-position the space layer (just in case)
                    layer_names_list[space_loop].set_offsets(0, 0)
		
                # Deselect
                pdb.gimp_selection_none(image)
				
            visi_loop = 0
            # Re-set the list of layer ids
            layer_names_list = image.layers
        
            while visi_loop < num_layers and other_layers > 0:
                visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
                visi_name = visi_name.decode('utf-8')
                if visi_loop <> first_letter_layer:
                    # turn off visibility of letter layers
                    pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)  
                visi_loop += 1
				
               
            # turn on visibility of character at the top of the layer stack and make it the active layer
            pdb.gimp_item_set_visible(layer_names_list[first_letter_layer],True)
            pdb.gimp_image_set_active_layer(image, layer_names_list[first_letter_layer])
				
        else:
            message_text = "There appears to be no suitable layer to create new letters from."
            old_handler = pdb.gimp_message_get_handler()	
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message_text)
            pdb.gimp_message_set_handler(old_handler)            
			
			
    if do_remove_non_letter_layers:
        
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
	
        visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
        do_not_remove = "legend"
        
        while visi_loop < num_layers:
            # check for layers to create new faces for
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if len(visi_name) == 1 or (len(visi_name) == 5 and ".png" in visi_name) or visi_name == "space.png" or do_not_remove.lower() in visi_name.lower():
			    # Re-position letter layers
                layer_names_list[visi_loop].set_offsets(0, 0)
            else:
			    # Remove other layers
                pdb.gimp_image_remove_layer(image, layer_names_list[visi_loop])
                pdb.gimp_image_resize_to_layers(image)
                    		
            visi_loop += 1
			

    if xcffilename == "": do_merge_files = False
	
    if do_merge_files:
        mode = fn_get_mode(old_mode, new_mode)
        font_name_pointer, file_dummy = -1, 0
        fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_list = image.layers
        loop = 0
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_list[loop])
            custom_layer = pdb.gimp_image_get_layer_by_name(fontimg, layer_name)
            if custom_layer >= 0: #layer found
                success = pdb.gimp_edit_copy(custom_layer)
                floating_sel = pdb.gimp_edit_paste(layer_list[loop], False)
                # set the layer mode as per the dialog UI when the contents of the original character layer have not been cleared
                if not clear_contents: pdb.gimp_layer_set_mode(floating_sel, mode)
                pdb.gimp_floating_sel_anchor(floating_sel)
            loop += 1
        
			
			
    if do_layer_info:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_list = image.layers
        loop = 0
        layer_info = ""
		
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_list[loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 5:
                layer_info += layer_name[0]
            elif (layer_name == "space.png"):
                layer_info += "space"
		
            loop += 1
        layer_info = layer_info.decode('utf-8') # Allows unicode characters above 127 to be displayed without creating errors
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(layer_info)
        pdb.gimp_message_set_handler(old_handler)
	
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
def fn_get_mode(old_mode, new_mode):
        # Use Legacy Modes if new Default Modes are set to Normal Mode
        temp_mode = 0
        if new_mode <> 0:
            if new_mode == 1:
                temp_mode = 1
            elif new_mode == 2:
                temp_mode = 57
            elif new_mode == 3:
                temp_mode = 59
            elif new_mode == 4:
                temp_mode = 60
            elif new_mode == 5:
                temp_mode = 36
            elif new_mode == 6:
                temp_mode = 55
            elif new_mode == 7:
                temp_mode = 31
            elif new_mode == 8:
                temp_mode = 42
            elif new_mode == 9:
                temp_mode = 33
            elif new_mode == 10:
                temp_mode = 35
            elif new_mode == 11:
                temp_mode = 54
            elif new_mode == 12:
                temp_mode = 30
            elif new_mode == 13:
                temp_mode = 43
            elif new_mode == 14:
                temp_mode = 53
            elif new_mode == 15:
                temp_mode = 23
            elif new_mode == 16:
                temp_mode = 45
            elif new_mode == 17:
                temp_mode = 44
            elif new_mode == 18:
                temp_mode = 48
            elif new_mode == 19:
                temp_mode = 49
            elif new_mode == 20:
                temp_mode = 50
            elif new_mode == 21:
                temp_mode = 51
            elif new_mode == 22:
                temp_mode = 32
            elif new_mode == 23:
                temp_mode = 52
            elif new_mode == 24:
                temp_mode = 34
            elif new_mode == 25:
                temp_mode = 46
            elif new_mode == 26:
                temp_mode = 47
            elif new_mode == 27:
                temp_mode = 41
            elif new_mode == 28:
                temp_mode = 37
            elif new_mode == 29:
                temp_mode = 38
            elif new_mode == 30:
                temp_mode = 39
            elif new_mode == 31:
                temp_mode = 40
            elif new_mode == 32:
                temp_mode = 24
            elif new_mode == 33:
                temp_mode = 25
            elif new_mode == 34:
                temp_mode = 26
            elif new_mode == 35:
                temp_mode = 27
            elif new_mode == 36:
                temp_mode = 56
            mode = temp_mode
        else:
            if old_mode == 1:
                temp_mode = 1
            elif old_mode == 2:
                temp_mode = 10
            elif old_mode == 3:
                temp_mode = 4
            elif old_mode == 4:
                temp_mode = 16
            elif old_mode == 5:
                temp_mode = 7
            elif old_mode == 6:
                temp_mode = 9
            elif old_mode == 7:
                temp_mode = 3
            elif old_mode == 8:
                temp_mode = 17
            elif old_mode == 9:
                temp_mode = 5
            elif old_mode == 10:
                temp_mode = 19
            elif old_mode == 11:
                temp_mode = 18
            elif old_mode == 12:
                temp_mode = 6
            elif old_mode == 13:
                temp_mode = 8
            elif old_mode == 14:
                temp_mode = 20
            elif old_mode == 15:
                temp_mode = 21
            elif old_mode == 16:
                temp_mode = 22
            elif old_mode == 17:
                temp_mode = 11
            elif old_mode == 18:
                temp_mode = 12
            elif old_mode == 19:
                temp_mode = 13
            elif old_mode == 20:
                temp_mode = 14
            else:
                temp_mode = 0
            mode = temp_mode
			

        return mode
	
def proc_trim_away(img, layer, amount):
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)


register(
    "python_fu_Custom_Font_Label_Setting_v1-09",
    "Provides options for custom font layer names",
    "Provides options for custom font layer names",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Labels v1.09...",
    "*",
    [
        (PF_TOGGLE, "random_bg_option", "Create new characters \nfrom visible layers:", 0),
        (PF_TOGGLE, "clear_face_option", "Clear contents of \noriginal character:", 0),
        (PF_OPTION, "mode_option", "Legacy Mode \nto use with font face:", 0, ("Normal", "Dissolve", "Lighten", "Screen", "Dodge", "Addition", "Darken", "Multiply", "Burn", "Overlay", "Softlight", "Hardlight", "Difference", "Subtract", "Grain Extract", "Grain Merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSV)")),
        (PF_OPTION, "mode_option2", "New Mode \nto use with font face:", 0, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
        (PF_TOGGLE, "remove", "Remove all non-font layers", 0),
        (PF_TOGGLE, "origin_option", "Ensure that all layers \nare positioned to the origin:", 0),
        (PF_TOGGLE, "sort_option", "Sort layers into order:", 0),
        (PF_TOGGLE, "info_option", "Display current layer names \nin a message box:", 0),
        (PF_STRING, "word", "Label String:", ".png"),
        (PF_TOGGLE, "layer_option", "Add Label String to \nsingle letter layer labels:", 0),
        (PF_TOGGLE, "path_option", "Add Label String to \nsingle letter path labels:", 0),
        (PF_TOGGLE, "pf_usecustomfont", "Combine the current character \nwith one from the custom font below:", 0),
        (PF_FILENAME, "pf_names_xcffilename", ".xcf custom font file:", 0),
    ],
    [],
    do_stuff)

main()

	

