#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
# It was inspired by MareroQ's filter Font-to-Layers
# MareroQ's much more elegant coding worked perfectly well
# except where the chosen font used left and/or right negative bearings.
# This resulted in some characters being clipped.
#
# My script takes a much simpler, dirtier approach and basically paints each character onto a layer two times and then merges the two layers
# to create a non-font layer.
#  
# The dialog can be accessed with Filters/Custom Font Layers from Fonts v0.02...
#
# The default setting will output Capital letters, numbers and keyboard punctuation and characters.
# You can optionally add lowercase characters
# and/or a set of special characters which are not available on the keyboard
# or even a subset of user selected characters
#
# Version 0.02 adds facility to adjust layer size to the right and left of each character (asked for by Alan via gimplearn.net)
# Version 0.03  restricts the layer height to the original font height (Python adds its own unicode default characters if they are not provided by the original font) and adds the options to adjust the layer height margins.
# Version 0.04 fixes a bug when using the restricted character set which crashed the filter when there was no 'A' character
# Version 0.041 Changed filter location to Custom Font Tools
#
# Current version 0.041
# 

from gimpfu import *

def do_the_stuff(image, layer, font_name, font_size, increase_margins, left_margin_size, right_margin_size, top_margin_size, bottom_margin_size, add_lower_case, add_specials, restricted_char_set, restricted_case):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    # check for unwanted space characters in input string
    restricted_case =  restricted_case.strip()
    if restricted_case == "": restricted_case = "321cbaCBA"
    if " " in restricted_case:
        loop, temp = 0, ""
        while loop < len(restricted_case):
            char = restricted_case[loop]
            if char <> " ": temp += char
            loop += 1
        if temp <> "":
            restricted_case = temp
        else:
           restricted_case = "321cbaCBA"
		   
    if restricted_char_set:
        layer_string = restricted_case
    else:
        upper_case = "ZYXWVUTSRQPONMLKJIHGFEDCBA" # "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        lower_case = "zyxwvutsrqponmlkjihgfedcba" #  "abcdefghijklmnopqrstuvwxyz"
        number_case = "0987654321" # "1234567890"
        other_case = "-=`[];'\,./~!@#$" + unichr(163) + "%^&*()_+{}:|<>?" # character 163 is the pound sterling sign
        special_case = unichr(169) + unichr(174) + unichr(8364) + unichr(188) + unichr(189) + unichr(190) + unichr(162) # Copyright, Registered Trademark, Euro sign, one quarter sign, one half sign, three quarters sign, cent sign
        special_case =  special_case + unichr(176) + unichr(185) + unichr(178) + unichr(179) + unichr(183) + unichr(215) + unichr(247) # degree sign, superscript 1, superscript 2, superscript 3, middle dot, multiplication, division
        layer_string = ""
        if add_specials: layer_string += special_case
        layer_string += (other_case + number_case)
        if add_lower_case: layer_string += lower_case
        layer_string += upper_case

    # find the maximum width and height of all layers
    max_width = 0
    max_height = 0
    (width, height, ascent, descent) = pdb.gimp_text_get_extents_fontname("A", font_size, 1, font_name)
    layer_height = height
    for char in layer_string:
        (width, height, ascent, descent) = pdb.gimp_text_get_extents_fontname(char, font_size, 1, font_name)
        if width > max_width:
            max_width = width
        if height > max_height:
            max_height = height
    # Add a token amount to both width and height to cater for unusual fonts
    if char == "A": layer_height = height
    max_width = max_width * 2
    max_height = max_height * 2
    # scale the image (and layers) to this new catch-all size
    pdb.gimp_image_scale(image, max_width,  max_height)
    # create layers from the characters in the layer string
    loop = 0
    if not increase_margins: 
        left_margin_size = 0.0
        right_margin_size = 0.0
        top_margin_size = 0.0
        bottom_margin_size = 0.0
    while loop < len(layer_string):
        # get the character
        char = layer_string[loop].decode('utf-8')
        # create the text layer from UI information at the origin of the image
        text_layer = pdb.gimp_text_fontname(image, None, 0.0 + left_margin_size, 0.0 + top_margin_size, char, 0, 1, font_size, 1, font_name)
        # create another copy of the same text layer
        text_layer = pdb.gimp_text_fontname(image, None, 0.0 + left_margin_size, 0.0 + top_margin_size, char, 0, 1, font_size, 1, font_name)
        # merge the two layer to throw away text information
        current_layer_width = text_layer.width
        current_layer_height = text_layer.height
        pdb.gimp_image_merge_down(image, text_layer, 0)
        # get the layer id by using the active layer status
        text_layer = pdb.gimp_image_get_active_layer(image)
        # add custom font suffix
        pdb.gimp_item_set_name(text_layer, char + ".png")
		
        if increase_margins:
            if (current_layer_width + (left_margin_size + right_margin_size)) < 10:
                new_layer_width = 10
            else:
                new_layer_width = current_layer_width + (left_margin_size + right_margin_size)

            pdb.gimp_layer_resize(text_layer, new_layer_width, layer_height + (top_margin_size + bottom_margin_size), left_margin_size, top_margin_size)
        else:
            if current_layer_height > layer_height:
                pdb.gimp_layer_resize(text_layer, current_layer_width, layer_height, 0, 0)
        loop += 1
		
		       
    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    loop = 0
        
    while loop < num_layers:
        # look for non-custom font layers
        name = pdb.gimp_item_get_name(layer_names_list[loop])
        name = name.decode('utf-8')
        if not((len(name) == 5 and ".png" in name) or name == "space.png"):
			    # Remove other layers
                pdb.gimp_image_remove_layer(image, layer_names_list[loop])                    		
        loop += 1
    # resize the image canvas to fit the custom font only layers
    pdb.gimp_image_resize_to_layers(image)
	
    
    # Find the width to use for the space character of the current font at the current size
    (width, height, ascent, descent) = pdb.gimp_text_get_extents_fontname(" ", font_size, 0, font_name)
    space_width = width
    im_height = image.height
    # create and insert the new space layer for the custom font using the image height
    space_layer = pdb.gimp_layer_new(image, space_width, im_height, 1, "space.png", 100, 0)
    pdb.gimp_image_insert_layer(image, space_layer, None, -1)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()

register(
    "python_fu_Layers_from_Fonts_v0-041",
    "Converts fonts to layers",
    "Converts fonts to layers",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Layers from Fonts v0.041...",
    "*",
    [
        (PF_FONT, "pf_font", "Font: ", "Sans"),
        (PF_SPINNER, "pf_fontsize", "Font size:", 200, (10, 3000, 1)),
        (PF_TOGGLE, "pf_increase_margins", "Adjust layer space around\nthe characters:", 0),
        (PF_SPINNER, "pf_leftmarginsize", "Adjustment (in pixels) to the \nleft of the character:", 0, (-1000, 1000, 1)),
        (PF_SPINNER, "pf_rightmarginsize", "Adjustment (in pixels) to the \nright of the character:", 0, (-1000, 1000, 1)),
        (PF_SPINNER, "pf_topmarginsize", "Adjustment (in pixels) to the \ntop of the character:", 0, (-1000, 1000, 1)),
        (PF_SPINNER, "pf_bottommarginsize", "Adjustment (in pixels) to the \nbottom of the character:", 0, (-1000, 1000, 1)),
        (PF_TOGGLE, "pf_add_lcase", "Add lower case layers:", 1),
        (PF_TOGGLE, "pf_add_specials", "Add special character layers:", 1),
        (PF_TOGGLE, "pf_restrict_set", "Create only the layers below:", 0),
        (PF_STRING, "pf_string", "Characters to use:", "321cbaCBA"),
    ],
    [],
    do_the_stuff)

main()